//
//  KakaoPartnerSDKSampleUITests.swift
//  KakaoPartnerSDKSampleUITests
//
//  Created by lucas on 2020/04/24.
//  Copyright © 2020 apiteam. All rights reserved.
//

import XCTest
import SBTUITestTunnelClient

import KakaoSDKCommon

class KakaoPartnerSDKSampleUITests: XCTestCase {
    var table: XCUIElementQuery {
        get {
            return self.app.tables
        }
    }
    
    override func setUp() {
        continueAfterFailure = false
        // 테스트 시 언어 설정 - 한국어 고정
        app.launchArguments = ["-AppleLanguages", "(ko)", "-AppleLocale", "ko_KR"]
        app.launchTunnel()
        
        // 테스트 데이터 스텁
        stubData(apiPath: Paths.authToken, module: "auth", method: "POST", host: "https://kauth.kakao.com")
        stubData(apiPath: Paths.userScopes, module: "user", method: "GET", fileName: "friends_scopes")
        stubData(apiPath: Paths.selectChats, module: "friends", method: "GET")
        stubData(apiPath: Paths.selectChatMembers, module: "friends", method: "GET")
        stubData(apiPath: Paths.selectFriends, module: "friends", method: "GET")
    }
    
    override func tearDown() {
        app.terminate()
    }
    
    func testSelectFriends() {
        let friendsTitle = "Friends Ui Test"
        
        createPicker(menuName: "PickerApi launcher - Friend") { [weak self] in
            let titleInput = self?.table.textFields["친구 선택"]
            titleInput?.tap()
            titleInput?.typeText("\(friendsTitle)\n")
            
            // 검색 비허용
            self?.table.cells.containing(.staticText, identifier: "검색 허용").children(matching: .button).element.tap()
        }
        
        // friend picker ui
        let title = app.staticTexts[friendsTitle]
        XCTAssertTrue(title.waitForExistence(timeout: 5))
        XCTAssertEqual(title.label, friendsTitle)
        
        let searchText = app.textFields.firstMatch
        XCTAssertFalse(searchText.exists)
        
        let okButtn = app.buttons["확인"]
        XCTAssertTrue(okButtn.exists)
        XCTAssertFalse(okButtn.isEnabled)
        
        table.cells["Tester2"].children(matching: .other).element(boundBy: 0).tap()
        table.cells["Tester3"].children(matching: .other).element(boundBy: 0).tap()
        table.cells["Tester4"].children(matching: .other).element(boundBy: 0).tap()
        table.cells["Tester4"].children(matching: .other).element(boundBy: 0).tap()
        
        let selectCnt = app.staticTexts["2"]
        XCTAssertTrue(selectCnt.exists)
        
        XCTAssertTrue(okButtn.isEnabled)
        okButtn.tap()
    }
    
    func testSelectChat() {
        let chatTitle = "Chat Ui Test"
        
        createPicker(menuName: "PickerApi launcher - Chat") { [weak self] in
            let titleInput = self?.table.textFields["친구 선택"]
            titleInput?.tap()
            titleInput?.typeText("\(chatTitle)\n")
        }
        
        // chat ui picker
        let title = app.staticTexts[chatTitle]
        XCTAssertTrue(title.waitForExistence(timeout: 5))
        XCTAssertEqual(title.label, chatTitle)
        
        let exitButton = app.buttons["닫기"]
        XCTAssertTrue(exitButton.exists)
        XCTAssertTrue(exitButton.isEnabled)
        
        let okButton = app.buttons["확인"]
        XCTAssertTrue(okButton.exists)
        XCTAssertFalse(okButton.isEnabled)
                
        table.cells["우리팀, 16명"].children(matching: .other).element(boundBy: 0).tap()
        table.cells["업무용, 13명"].children(matching: .other).element(boundBy: 0).tap()
        table.cells["잡담방, 300명, 오픈채팅"].children(matching: .other).element(boundBy: 0).tap()
        
        XCTAssertTrue(okButton.isEnabled)
        okButton.tap()
    }
    
    func testSelectChatMember() {
        let chatTitle = "Select Chat Member Test"
                
        createPicker(menuName: "PickerApi launcher - Chat") { [weak self] in
            let titleInput = self?.table.textFields["친구 선택"]
            titleInput?.tap()
            titleInput?.typeText("\(chatTitle)\n")
            
            self?.table.cells.containing(.staticText, identifier:"챗방/챗맴버 피커 타입").buttons["NULL"].tap()
            self?.app.scrollViews.otherElements.buttons["chatMember"].tap()
        }
        
        // chat ui
        let title = app.staticTexts[chatTitle]
        XCTAssertTrue(title.waitForExistence(timeout: 5))
        XCTAssertEqual(title.label, chatTitle)
        
        let exitButton = app.buttons["닫기"]
        XCTAssertTrue(exitButton.exists)
        XCTAssertTrue(exitButton.isEnabled)
        
        let nextButton = app.buttons["다음"]
        XCTAssertTrue(nextButton.exists)
        XCTAssertFalse(nextButton.isEnabled)
        
        table.cells["우리팀, 16명"].children(matching: .other).element(boundBy: 0).tap()
        XCTAssertTrue(nextButton.isEnabled)
        nextButton.tap()
        
        // chat member ui
        let prevButton = app.buttons["이전"]
        XCTAssertTrue(prevButton.exists)
        XCTAssertTrue(prevButton.isEnabled)
        
        let okButton = app.buttons["확인"]
        XCTAssertTrue(okButton.exists)
        XCTAssertFalse(okButton.isEnabled)
        
        let selectAll = app.buttons["전체 선택"]
        XCTAssertTrue(selectAll.exists)
        XCTAssertTrue(selectAll.isEnabled)
        
        let deSelectAll = app.buttons["전체 해제"]
        XCTAssertFalse(deSelectAll.exists)
        
        table.cells["Tester1"].children(matching: .other).element(boundBy: 0).tap()
        selectAll.tap()
        XCTAssertTrue(deSelectAll.isEnabled)
        deSelectAll.tap()
        table.cells["Tester2"].children(matching: .other).element(boundBy: 0).tap()
        
        XCTAssertTrue(okButton.isEnabled)
        okButton.tap()
    }
    
    func testTabPicker() {
        createPicker(menuName: "PickerApi launcher - Tab")
        
        // tab picker ui
        let titleString = "친구 선택"
        let title = app.staticTexts[titleString]
        XCTAssertTrue(title.waitForExistence(timeout: 3))
        
        let exitButton = app.buttons["닫기"]
        XCTAssertTrue(exitButton.exists)
        XCTAssertTrue(exitButton.isEnabled)
        
        let okButtn = app.buttons["확인"]
        XCTAssertTrue(okButtn.exists)
        XCTAssertFalse(okButtn.isEnabled)
        
        let friendTab = app.buttons["친구"]
        XCTAssertTrue(friendTab.exists)
        XCTAssertTrue(friendTab.isEnabled)
        
        let chatTab = app.buttons["채팅"]
        XCTAssertTrue(chatTab.exists)
        XCTAssertTrue(chatTab.isEnabled)
        
        chatTab.tap()
        table.cells["우리팀, 16명"].children(matching: .other).element(boundBy: 0).tap()
        XCTAssertTrue(okButtn.isEnabled)
        
        friendTab.tap()
        XCTAssertFalse(okButtn.isEnabled)
        
        table.cells["Tester1"].children(matching: .other).element(boundBy: 0).tap()
        XCTAssertTrue(okButtn.isEnabled)
        okButtn.tap()
    }
    
    func testPickerRotate() {
        createPicker(menuName: "PickerApi launcher - Friend")
        
        // friend picker ui
        table.cells["Tester2"].children(matching: .other).element(boundBy: 0).tap()
        
        let selectCnt = app.staticTexts.element(boundBy: 1)
        XCTAssertTrue(selectCnt.exists)
        XCTAssertEqual("1", selectCnt.label)
        
        // rotate
        XCUIDevice.shared.orientation = .landscapeLeft
        XCTAssertTrue(selectCnt.exists)
        XCTAssertEqual("1", selectCnt.label)
                
        XCUIDevice.shared.orientation = .portrait
        app.buttons["확인"].tap()
    }
    
    func testFriendsMsgBlocked() {
        createPicker(menuName: "PickerApi launcher - Friend") { [weak self] in
            // 내 프로필 없음
            self?.table.cells.containing(.staticText, identifier: "내 프로필 보기").children(matching: .button).element(boundBy: 0).tap()
            // 등록된 유저에 대해 메시지 블록
            self?.table.staticTexts["Registered"].tap()
        }
        
        // friend picker ui
        let okButton = app.buttons["확인"]
        XCTAssertTrue(okButton.exists)
        XCTAssertFalse(okButton.isEnabled)
        
        let me = table.cells["Tester1, 가입"].children(matching: .other).element(boundBy: 0)
        XCTAssertFalse(me.exists)
        
        let blockedFriend = table.cells["Tester2, 가입"].children(matching: .other).element(boundBy: 0)
        XCTAssertTrue(blockedFriend.exists)
        blockedFriend.tap()
        
        XCTAssertFalse(okButton.isEnabled)
    }
    
    func testFilterOs() {
        createPicker(menuName: "PickerApi launcher - Friend") { [weak self] in
            self?.table.cells.containing(.staticText, identifier: "친구 디바이스 OS").buttons["NULL"].tap()
            self?.app.scrollViews.otherElements.buttons["iOS"].tap()
        }
        
        // friend picker ui
        
        // android user
        let tester2 = table.cells["Tester2"].children(matching: .other).element(boundBy: 0)
        XCTAssertFalse(tester2.exists)
        let tester4 = table.cells["Tester4"].children(matching: .other).element(boundBy: 0)
        XCTAssertFalse(tester4.exists)
        
        // iOS user
        let tester3 = table.cells["Tester3"].children(matching: .other).element(boundBy: 0)
        XCTAssertTrue(tester3.exists)
        XCTAssertTrue(tester3.isEnabled)
        tester3.tap()
        
        app.buttons["확인"].tap()
    }
    
    func testSearch() {
        createPicker(menuName: "PickerApi launcher - Friend")
        
        // picker ui
        let searchTextField = app.textFields["이름, 닉네임 검색"]
        XCTAssertTrue(searchTextField.exists)
        XCTAssertTrue(searchTextField.isEnabled)
        
        searchTextField.tap()
        searchTextField.typeText("aaaa")
        
        let noResult = table["검색 결과가 없습니다."]
        XCTAssertTrue(noResult.exists)
                
        searchTextField.doubleTap()
        searchTextField.typeText("Tester")
                
        XCTAssertEqual(table.cells.allElementsBoundByIndex.count, 4)
    }
    
    func testSelectMaxFriends() {
        let maxSelect = 1
        
        createPicker(menuName: "PickerApi launcher - Friend") { [weak self] in
            let maxSetting = self?.table.textFields["30"]
            maxSetting?.tap()
            maxSetting?.typeText("\(maxSelect)\n")
        }
        
        // friend picker ui
        table.cells["Tester2"].children(matching: .other).element(boundBy: 0).tap()
        table.cells["Tester3"].children(matching: .other).element(boundBy: 0).tap()
        
        addUIInterruptionMonitor(withDescription: "Test Max") { (element) -> Bool in
            if element.containsString(value: "친구는 한 번에") {
                element.buttons["OK"].tap()
                return true
            }
            return false
        }
        
        app.tap()
        
        let selectedUser = app.collectionViews.cells
        XCTAssertEqual(selectedUser.allElementsBoundByIndex.count, maxSelect)
        
        app.buttons["확인"].tap()
    }
    
    func testSelectMinFriends() {
        let minSelect = 2
        
        createPicker(menuName: "PickerApi launcher - Friend") { [weak self] in
            let minSetting = self?.table.textFields["1"]
            minSetting?.tap()
            minSetting?.typeText("\(minSelect)\n")
        }
        
        let selectedUsers = app.collectionViews.cells
        
        // friend picker ui
        table.cells["Tester1"].children(matching: .other).element(boundBy: 0).tap()
        XCTAssertEqual(selectedUsers.allElementsBoundByIndex.count, 1)
        app.buttons["확인"].tap()
        
        addUIInterruptionMonitor(withDescription: "Test Min") { (element) -> Bool in
            if element.containsString(value: "최소 ") {
                element.buttons["OK"].tap()
                return true
            }
            return false
        }
        
        app.tap()
        
        table.cells["Tester2"].children(matching: .other).element(boundBy: 0).tap()
        XCTAssertEqual(selectedUsers.allElementsBoundByIndex.count, minSelect)
        app.buttons["확인"].tap()
    }
}

extension KakaoPartnerSDKSampleUITests {
    private func createPicker(menuName: String, pickerSettings: (() -> Void)? = nil) {
        app.tabBars.firstMatch.buttons["KakaoPartnerSDK"].tap()
        table.staticTexts[menuName].tap()
        
        pickerSettings?()
        
        let createBtn = app.buttons.element(boundBy: 1)
        XCTAssertTrue(createBtn.waitForExistence(timeout: 5))
        XCTAssertEqual(createBtn.label, "피커 생성")
        createBtn.tap()
    }
}

extension XCUIElement {
    func containsString(value: String) -> Bool {
        let predicate = NSPredicate(format: "label contains %@", value)
        return self.staticTexts.containing(predicate).firstMatch.exists
    }
}
